@@ -11,6 +11,7 @@ PATH_PRJ = '/'.join(os.path.abspath(__file__).split('/')[:-2])
 sys.path.append(PATH_PRJ)
 
 """
+from copy import deepcopy
 import ahocorasick
 import csv
 import json
@@ -215,6 +216,76 @@ def save_json_list_file(data, path_tgt):
 '''excel'''
 
 
+def load_xlsx(path_src, st=[], return_all=False):
+    """读取 .xlsx 数据
+
+    Args:
+        path_src (str): 文件路径
+        st (list): sheet 名，返回指定了名字的 sheet 的内容，默认返回第一个 sheet
+        return_all (bool): 是否返回所有 sheet 的数据
+    Return:
+        dict: { <sheet_name>: <list of data> }
+            如果某个 sheet 数据为空，则不会返回该 sheet 数据
+    """
+    wb = xlrd.open_workbook(path_src)
+    sheet2st = {}
+    if st:
+        for sheet in st:
+            try:
+                st = wb.sheet_by_name(sheet)
+                sheet2st[sheet] = st
+            except Exception:
+                sheet2st[sheet] = None
+    elif return_all is True:
+        sheet2st = {x.name: x for x in wb.sheets() if x.nrows > 0}
+    else:
+        st = wb.sheets()[0]
+        sheet2st[st.name] = st
+
+    sheet2data = {}
+    for k, st in sheet2st.items():
+        nrows = st.nrows
+        ret = []
+        for i in range(nrows):
+            row = [pre_process_int(x) for x in st.row_values(i)]
+            ret.append(row)
+        sheet2data[k] = ret
+    return sheet2data
+
+
+def save_xlsx(data, path_tgt, st_names=[], titles=[]):
+    """保存数据至 xlsx
+
+    Args:
+        data (list): 要存储的数据，三重列表 sheet - row - column
+        path_tgt (str):
+        st_names (list): sheet的名字，默认 sheet1 sheet2 ...
+        titles (list): 如果没有给定，则 data 从第一行开始存储；如果指定，需要对每个 sheet 都指定
+    """
+    if not st_names:
+        st_names = [f'sheet{i+1}' for i in range(len(data))]
+    else:
+        assert len(st_names) == len(data), 'st_names 必须和 data 的长度一致'
+    if not titles:
+        titles = [[] for _ in range(len(data))]
+    else:
+        assert len(titles) == len(data), 'titles 必须和 data 的长度一致'
+
+    wb = xlsxwriter.Workbook(path_tgt)
+    for sheet_data, st_name, title in zip(data, st_names, titles):
+        st = wb.add_worksheet(st_name)
+        if title:
+            for i, item in enumerate(title):
+                st.write(0, i, item)
+            start_id = 1
+        else:
+            start_id = 0
+        for i, row_values in enumerate(sheet_data):
+            for j, item in enumerate(row_values):
+                st.write(i+start_id, j, str(item))
+    wb.close()
+
+
 def load_wb(path_src):
     """
     - sheets()
@@ -293,6 +364,35 @@ def save_wb(data, path_tgt, st_name='default', title=None):
     wb.close()
 
 
+def get_table_lines_markdown(title, data, add_first_id=False):
+    """
+    
+    Args:
+        title (list[str]): ['列1', '列2']
+        data (list[list]): [ [1, 2] ]
+        add_first_id (bool): 是否在第一列加 id（下标从1开始）
+    Returns:
+        list: 可直接写入 markdown 文件的格式
+            '|列1 | 列2|'
+            '|---|---|'
+            '|1|2|'
+    """
+    title = deepcopy(title)
+    lines = []
+    lines.append(title)
+    lines.append(['-'] * len(title))
+    lines.extend(data)
+
+    # 第一列加 id
+    if add_first_id is True:
+        lines[0].insert(0, 'id')
+        lines[1].insert(0, '-')
+        for i, line in enumerate(lines[2:]):
+            line.insert(0, str(i+1))
+    lines = ['|' + ' | '.join(x) + '|' for x in lines]
+    return lines
+
+
 '''csv'''
 
 
